"""
SMOS L3 Ascending and Descending, daily products, opearive and reprocessed, (A:ascending=6am and D:descending=6pm)

Downloaded from CATDS:

ftp.ifremer.fr
user : ext-catds-cpdc
p : catds2010

on 09-05-2022

Note:

* OPER(ative) and REP(rocessed) are mixed in the MIR_CLF3SA (Ascending) and MIR_CLF3SD (Descending) folders
"""